from pyftg.ai_interface import AIInterface
from model import RecurrentActor, RecurrentCritic
from pyftg.struct import *
import numpy as np
import torch

#Based on: https://github.com/TeamFightingICE/BlindAI/blob/master/agent.py
#TODO: Write in a way that is easier to implement different models

STATE_DIM = {
    1: {
        'conv1d': 160,
        'fft': 512,
        'mel': 2560
    },
    4: {
        'conv1d': 64,
        'fft': 512,
        'mel': 1280
    }
}

HIDDEN_SIZE = 512
RECURRENT_LAYERS = 1
LEARNING_RATE = 0.0003
GAMMA = 0.99
C1 = 0.95
LAMBDA = 0.95
BATCH_SIZE = 64

class SoundAgent(AIInterface):
    def __init__(self):
        super().__init__()
        self.blind_flag = True
        self.nonDelay: FrameData = None
        self.pre_framedata: FrameData = None
        self.audio_data = None
        self.raw_audio_memory = None 
        self.n_frame = 1
        self.just_inited = True
        self.actor = RecurrentActor(STATE_DIM[1]['mel'], HIDDEN_SIZE, RECURRENT_LAYERS,
                        action_num=40)
        self.critic = RecurrentCritic(STATE_DIM[1]['mel'], HIDDEN_SIZE, RECURRENT_LAYERS)
        self.actor.get_init_state('cpu')
        self.critic.get_init_state('cpu')
        self.actions = "AIR_A", "AIR_B", "AIR_D_DB_BA", "AIR_D_DB_BB", "AIR_D_DF_FA", "AIR_D_DF_FB", "AIR_DA", "AIR_DB", \
                       "AIR_F_D_DFA", "AIR_F_D_DFB", "AIR_FA", "AIR_FB", "AIR_UA", "AIR_UB", "BACK_JUMP", "BACK_STEP", \
                       "CROUCH_A", "CROUCH_B", "CROUCH_FA", "CROUCH_FB", "CROUCH_GUARD", "DASH", "FOR_JUMP", "FORWARD_WALK", \
                       "JUMP", "NEUTRAL", "STAND_A", "STAND_B", "STAND_D_DB_BA", "STAND_D_DB_BB", "STAND_D_DF_FA", \
                       "STAND_D_DF_FB", "STAND_D_DF_FC", "STAND_F_D_DFA", "STAND_F_D_DFB", "STAND_FA", "STAND_FB", \
                       "STAND_GUARD", "THROW_A", "THROW_B"

    def name(self) -> str:
        return self.__class__.__name__

    def is_blind(self) -> bool:
        return self.blind_flag

    def initialize(self, game_data: GameData, player_number: int):
        self.cc = CommandCenter()
        self.key = Key()
        self.player = player_number
        self.frameData = FrameData()

    def input(self) -> Key:
        return self.key

    def get_information(self, frame_data: FrameData, is_control: bool, non_delay_frame_data: FrameData):
        self.frame_data = frame_data
        self.cc.set_frame_data(self.frame_data, self.player)
        self.pre_framedata = self.nonDelay if self.nonDelay is not None else non_delay_frame_data
        self.nonDelay = non_delay_frame_data
        self.isControl = is_control
        self.currentFrameNum = self.frameData.current_frame_number

    def get_screen_data(self, screen_data: ScreenData):
        self.screen_data = screen_data

    def get_audio_data(self, audio_data: AudioData):
        self.audio_data = audio_data
        try:
            byte_data = self.audio_data.raw_data_as_bytes
            np_array = np.frombuffer(byte_data, dtype=np.float32)
            raw_audio = raw_audio.T
            raw_audio = raw_audio[:800, :]
        except Exception as ex:
            raw_audio = np.zeros((800, 2))
        if self.raw_audio_memory is None:
            self.raw_audio_memory = raw_audio
        else:
            self.raw_audio_memory = np.vstack((raw_audio, self.raw_audio_memory))
            self.raw_audio_memory = self.raw_audio_memory[:800 * self.n_frame, :]
        increase = (800 * self.n_frame - self.raw_audio_memory.shape[0]) // 800
        for _ in range(increase):
            self.raw_audio_memory = np.vstack((np.zeros((800, 2)), self.raw_audio_memory))

    def processing(self):
        if self.frame_data.empty_flag or self.frame_data.current_frame_number <= 0:
            return

        self.key.empty()
        self.cc.skill_cancel()

        obs = self.raw_audio_memory
        if self.just_inited:
            self.just_inited = False
            if obs is None:
                obs = np.zeros((800 * self.n_frame, 2))
            terminal = 1
        elif obs is None:
            obs = np.zeros((800 * self.n_frame, 2))
        else:
            terminal = 0

        state = torch.tensor(obs, dtype=torch.float32)
        action_dist = self.actor(state.unsqueeze(0), terminal=torch.tensor(terminal).float())
        action = action_dist.sample()
        self.cc.command_call(self.actions[action])
        self.key = self.cc.get_skill_key()
    
    def round_end(self, round_result: RoundResult):
        print(round_result.remaining_hps[0])
        print(round_result.remaining_hps[1])
        print(round_result.elapsed_frame)

    def game_end(self):
        pass


